#pragma once

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <string>
#include <vector>
#include <errno.h>
#include <sys/stat.h>
#ifdef _WIN32
#include <direct.h>
#endif

#include "minizip/unzip.h"
#include "GakumasLocalify/Log.h"

// ݹ鴴Ŀ¼
static bool CreateDirectoryRecursively(const std::string& dir) {
    if (dir.empty())
        return false;

    // ԴĿ¼
#ifdef _WIN32
    int ret = _mkdir(dir.c_str());
#else
    int ret = mkdir(dir.c_str(), 0755);
#endif
    if (ret == 0 || errno == EEXIST)
        return true;

    // ʧܣȴĿ¼
    size_t pos = dir.find_last_of("/\\");
    if (pos != std::string::npos) {
        std::string parentDir = dir.substr(0, pos);
        if (!CreateDirectoryRecursively(parentDir))
            return false;
#ifdef _WIN32
        ret = _mkdir(dir.c_str());
#else
        ret = mkdir(dir.c_str(), 0755);
#endif
        return (ret == 0 || errno == EEXIST);
    }
    return false;
}

// ѹ
static bool UnzipFile(const std::string& zipPath, const std::string& destinationFolder) {
    // zipļ
    unzFile zipfile = unzOpen(zipPath.c_str());
    if (zipfile == nullptr) {
        GakumasLocal::Log::ErrorFmt( "Can't open zip file: %s", zipPath.c_str());
        return false;
    }

    int ret = unzGoToFirstFile(zipfile);
    if (ret != UNZ_OK) {
        GakumasLocal::Log::ErrorFmt("Can't read first file %s", zipPath.c_str());
        unzClose(zipfile);
        return false;
    }

    // zipڵÿļ
    do {
        char filename[512] = { 0 };
        unz_file_info fileInfo;
        ret = unzGetCurrentFileInfo(zipfile, &fileInfo,
            filename, sizeof(filename),
            nullptr, 0, nullptr, 0);
        if (ret != UNZ_OK) {
            GakumasLocal::Log::ErrorFmt("Read ZIP File Info Error");
            unzClose(zipfile);
            return false;
        }

        std::string filePath = filename;
        std::string fullPath = destinationFolder;
        // ֤ĿĿ¼·ָβ
        if (fullPath.back() != '/' && fullPath.back() != '\\') {
            fullPath += "/";
        }
        fullPath += filePath;

        // жǷΪĿ¼Ŀ¼Ŀͨ'/'β
        if (!filePath.empty() && (filePath.back() == '/' || filePath.back() == '\\')) {
            // Ŀ¼
            if (!CreateDirectoryRecursively(fullPath)) {
                GakumasLocal::Log::ErrorFmt("Create Dir Failed: %s", fullPath.c_str());
                unzClose(zipfile);
                return false;
            }
        }
        else {
            // ļȷϼĿ¼
            size_t pos = fullPath.find_last_of("/\\");
            if (pos != std::string::npos) {
                std::string directory = fullPath.substr(0, pos);
                if (!CreateDirectoryRecursively(directory)) {
                    GakumasLocal::Log::ErrorFmt("Create Dir Failed: %s", directory.c_str());
                    unzClose(zipfile);
                    return false;
                }
            }

            // zipļ
            ret = unzOpenCurrentFile(zipfile);
            if (ret != UNZ_OK) {
                GakumasLocal::Log::ErrorFmt("Open file in zip failed: %s", filePath.c_str());
                unzClose(zipfile);
                return false;
            }

            // Ŀ·ϴļ
            FILE* outFile = fopen(fullPath.c_str(), "wb");
            if (outFile == nullptr) {
                GakumasLocal::Log::ErrorFmt("Can't create output file: %s", fullPath.c_str());
                unzCloseCurrentFile(zipfile);
                unzClose(zipfile);
                return false;
            }

            // ȡݲдļ
            const int bufferSize = 8192;
            std::vector<char> buffer(bufferSize);
            int bytesRead = 0;
            do {
                bytesRead = unzReadCurrentFile(zipfile, buffer.data(), bufferSize);
                if (bytesRead < 0) {
                    GakumasLocal::Log::ErrorFmt("Read File Error: %s", filePath.c_str());
                    fclose(outFile);
                    unzCloseCurrentFile(zipfile);
                    unzClose(zipfile);
                    return false;
                }
                if (bytesRead > 0) {
                    fwrite(buffer.data(), 1, bytesRead, outFile);
                }
            } while (bytesRead > 0);

            fclose(outFile);
            unzCloseCurrentFile(zipfile);
        }

        ret = unzGoToNextFile(zipfile);
    } while (ret == UNZ_OK);

    unzClose(zipfile);
    return true;
}
